/* --------------------------------------------------------------------------------
 * EXAMPLE.C
 *
 * 04/2019 : F. Prill, DWD
 *
 * limitations:
 *
 * - the remainder in the mtime struct "_divisionquotienttimespan" is
 *   limited to int64_t msecs.
 *
 * -------------------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>
#include "int_div.h"
#include <gmp.h>

int divide_timespan_mpz(const t_int days1, const t_int day_fraction1,
                        const t_int days2, const t_int day_fraction2,
                        t_int *q_decimal, t_int *remainder_days, t_int *remainder_ms);

int main(void) {
  const int NREPEAT = 3000000;

  int   ret, i, cnt = 0;
  t_int days1, day_fraction1, days2, day_fraction2;
  t_int q_decimal0 = 0, remainder_days0 = 0, remainder_ms0 = 0;
  t_int q_decimal  = 0, remainder_days  = 0, remainder_ms  = 0;

  double f1 = 1728000000.0 / (double) RAND_MAX;
  double f2 = 86400000.0   / (double) RAND_MAX;

  printf("test program\n");
  printf("  checking %d random divisions against GNU Multiple Precision Arithmetic Library\n",NREPEAT);
  srand ( 123 );

  for (i=0; i<NREPEAT; i++) {
    /* test randomly generated input against GNU Multiple Precision
       Arithmetic Library: */
    days1           = (t_int) ( f1 * (double) rand() );
    day_fraction1   = (t_int) ( f2 * (double) rand() );
    days2           = (t_int) ( f1 * (double) rand() );
    day_fraction2   = (t_int) ( f2 * (double) rand() );

    ret = divide_timespan_mpz(days1, day_fraction1, days2, day_fraction2,
                              &q_decimal0, &remainder_days0, &remainder_ms0);
    ret = divide_timespan(days1, day_fraction1, days2, day_fraction2,
                          &q_decimal, &remainder_days, &remainder_ms);

    if ((ret != 0)                          ||
        (q_decimal != q_decimal0)           ||
        (remainder_days != remainder_days0) ||
        (remainder_ms != remainder_ms0))
      {
        printf("dividend: %" PRId64 " : %" PRId64 " \n", days1, day_fraction1);
        printf("divisor : %" PRId64 " : %" PRId64 " \n", days2, day_fraction2);
        printf("comparison: quotient_out = %" PRId64 ", remainder = %" PRId64 " : %" PRId64 "\n",
               q_decimal0, remainder_days0, remainder_ms0);
        printf("quotient_out = %" PRId64 ", remainder = %" PRId64 " : %" PRId64 "\n",
               q_decimal, remainder_days, remainder_ms);
        printf("return code = %d\n",ret);
        cnt++;
      }
  }

  printf("  %d discrepancies reported.\n", cnt);
  printf("test program done after %d tests.\n", NREPEAT);
  exit(0);
}


int divide_timespan_mpz(const t_int days1, const t_int day_fraction1,
                        const t_int days2, const t_int day_fraction2,
                        t_int *q_decimal, t_int *remainder_days, t_int *remainder_ms)
{
  mpz_t a0, a1, a2, b1, b2, q, r;

  mpz_inits(a0,a1,a2,b1,b2,q,r,NULL);

  mpz_set_ui(a0, 86400000);
  mpz_set_ui(a1,days1);
  mpz_set_ui(a2,day_fraction1);
  mpz_set_ui(b1,days2);
  mpz_set_ui(b2,day_fraction2);

  mpz_addmul(a2,a1,a0);
  mpz_addmul(b2,b1,a0);
  mpz_fdiv_qr(q,r,a2,b2);

  *q_decimal = mpz_get_ui(q);

  mpz_fdiv_qr(a2,b2,r,a0);
  *remainder_days = mpz_get_ui(a2);
  *remainder_ms   = mpz_get_ui(b2);

  mpz_clears(a0,a1,a2,b1,b2,q,r,NULL);
  return 0;
}
